'''
    twiz - Your Twitter Account Data Analysis & Visualization Tool <3

    twiz can help you in analyzing and visualizing your twitter account's
    exported data offline. 

    Author: Anjan Roy <anjanroy@yandex.com>
    License: CC0-1.0
'''
__version__ = '0.3.0'

from warnings import filterwarnings

filterwarnings('ignore')



#!/usr/bin/python3

import seaborn as sns
from matplotlib import pyplot as plt
from typing import List, Tuple
from copy import deepcopy
from ..model.manip import (
    getFollowersAndFollowingsPerCent,
    getFollowedFollowers
)
from ..model.like import (
    topXHashTagsInLikedTweets,
    topXTaggedUsersInLikedTweets,
    topXEmojisInLikedTweets
)
from emoji import demojize
from textwrap import wrap


def plotFollowersAndFollowings(followers: map, followings: map, title: str, sink: str) -> bool:
    '''
        Plotting follower & following data as bar chart
    '''
    try:
        _data = getFollowersAndFollowingsPerCent(deepcopy(followers),
                                                 deepcopy(followings))
        _followerCount = len(list(map(lambda e: e[0],
                                      deepcopy(followers))))
        _followingCount = len(list(map(lambda e: e[0],
                                       deepcopy(followings))))

        fig = plt.Figure(figsize=(16, 9), dpi=100)

        sns.set_style('darkgrid')

        sns.barplot(x=['Followers', 'Followings'], y=_data, ax=fig.gca())

        for i, j in enumerate(fig.gca().patches):
            fig.gca().text(j.get_x() + j.get_width() * .5,
                           j.get_y() + j.get_height() * .5,
                           ['Count: {} [ {:.2f}% ]'.format(_followerCount, _data[0]),
                            'Count: {} [ {:.2f}% ]'.format(_followingCount, _data[1])][i],
                           ha='center',
                           rotation=0,
                           fontsize=14,
                           color='black')

        fig.gca().set_ylim(0, 100)
        fig.gca().set_title(title, fontsize=22, pad=10)

        fig.savefig(sink, pad_inches=.5)
        plt.close(fig)

        return True
    except Exception:
        return False


def plotFollowersAndFollowedFollowers(followers: map, followings: map, title: str, sink: str) -> bool:
    '''
        Plotting followers & followed followers ( i.e.
        followers whom you follow back ) data as bar chart
    '''
    try:
        _followerCount = len(list(map(lambda e: e[0], deepcopy(followers))))
        _followingCount = len(list(map(lambda e: e[0], deepcopy(followings))))
        _commonCount = len(getFollowedFollowers(deepcopy(followers),
                                                deepcopy(followings)))

        fig, axes = plt.subplots(1, 2,
                                 sharey=True,
                                 figsize=(16, 9),
                                 dpi=100)
        sns.set_style('darkgrid')

        sns.barplot(x=['Followers',
                       'Followers whom YOU followed back'],
                    y=[100, (_commonCount * 100) / _followerCount],
                    ax=axes[0])

        for i, j in enumerate(axes[0].patches):
            axes[0].text(j.get_x() + j.get_width() * .5,
                         j.get_y() + j.get_height() * .5,
                         [_followerCount, _commonCount][i],
                         ha='center',
                         rotation=0,
                         fontsize=14,
                         color='black')

        sns.barplot(x=['Followings',
                       'Followings who\'re following YOU'],
                    y=[100, (_commonCount * 100) / _followingCount],
                    ax=axes[1])

        for i, j in enumerate(axes[1].patches):
            axes[1].text(j.get_x() + j.get_width() * .5,
                         j.get_y() + j.get_height() * .5,
                         [_followingCount, _commonCount][i],
                         ha='center',
                         rotation=0,
                         fontsize=14,
                         color='black')

        axes[0].set_ylim(0, 100)
        axes[1].set_ylim(0, 100)
        axes[0].set_title(title[0], pad=4)
        axes[1].set_title(title[1], pad=4)

        fig.savefig(sink, pad_inches=.5)
        plt.close(fig)

        return True
    except Exception:
        return False


def plotTopXHashTagsFoundInLikedTweets(likes: map, x: int, title: str, sink: str) -> bool:
    '''
        Plotting top X most found hash tags in tweets liked by YOU as bar plot
    '''
    try:
        _data = topXHashTagsInLikedTweets(deepcopy(likes), x)
        _x, _y = [], []
        for i in _data:
            _x.append(i[0])
            _y.append(i[1])

        fig = plt.Figure(figsize=(16, 9), dpi=100)

        sns.set_style('darkgrid')

        sns.barplot(x=_x, y=_y, ax=fig.gca())

        for i, j in enumerate(fig.gca().patches):
            fig.gca().text(j.get_x() + j.get_width() * .5,
                           j.get_y() + j.get_height() * .5,
                           _y[i],
                           ha='center',
                           rotation=0,
                           fontsize=15,
                           color='black')

        fig.gca().set_title(title, fontsize=22, pad=10)

        fig.savefig(sink, pad_inches=.5)
        plt.close(fig)

        return True
    except Exception:
        return False


def plotTopXTaggedUsersFoundInLikedTweets(likes: map, x: int, title: str, sink: str) -> bool:
    '''
        Plotting top X most found tagged user names in tweets liked by YOU as bar plot
    '''
    try:
        _data = topXTaggedUsersInLikedTweets(deepcopy(likes), x)
        _x, _y = [], []
        for i in _data:
            _x.append(i[1])
            _y.append(i[0])

        fig = plt.Figure(figsize=(16, 9), dpi=100)
        sns.set_style('darkgrid')

        sns.barplot(x=_x, y=_y, ax=fig.gca(), orient='h')

        for i, j in enumerate(fig.gca().patches):
            fig.gca().text(j.get_x() + j.get_width() * .5,
                           j.get_y() + j.get_height() * .5,
                           _x[i],
                           ha='center',
                           rotation=0,
                           fontsize=16,
                           color='black')

        fig.gca().set_title(title, fontsize=22, pad=10)

        fig.savefig(sink, pad_inches=.5)
        plt.close(fig)

        return True
    except Exception:
        return False


def plotTopXEmojisFoundInLikedTweets(likes: map, x: int, title: str, sink: str) -> bool:
    '''
        Plotting top X most found emojis ( in demojized form i.e. text 
        based representation ) in tweets liked by YOU as bar plot
    '''
    try:
        _data = topXEmojisInLikedTweets(deepcopy(likes), x)
        _x, _y = [], []
        for i in _data:
            _x.append(i[1])
            _y.append(i[0])

        fig = plt.Figure(figsize=(16, 9), dpi=100)
        sns.set_style('darkgrid')

        sns.barplot(x=_x,
                    y=['\n'.join(wrap(' '.join([j.capitalize() for j in demojize(
                        i).strip(':').split('_')]), width=16)) for i in _y],
                    ax=fig.gca(), orient='h')

        for i, j in enumerate(fig.gca().patches):
            fig.gca().text(j.get_x() + j.get_width() * .5,
                           j.get_y() + j.get_height() * .5,
                           _x[i],
                           ha='center',
                           rotation=0,
                           fontsize=16,
                           color='black')

        fig.gca().set_title(title, fontsize=22, pad=10)
        fig.tight_layout(pad=4)

        fig.savefig(sink, pad_inches=.5)
        plt.close(fig)

        return True
    except Exception:
        return False


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

from typing import List, Tuple, Dict, Any
from re import compile as regCompile, I as regI, Pattern
from itertools import chain
from collections import Counter
from emoji import emoji_lis


def getLikes(data: List[Dict[str, Any]]) -> map:
    '''
        Extract likes & returns stream of tuples, where each of the tuples are
        going to be of form ('id', 'text', 'url')
    '''

    def _helper(obj: Dict[str, Any]) -> Tuple[str, str, str]:
        _like = obj['like']

        return _like['tweetId'], _like['fullText'], _like['expandedUrl']

    return map(_helper, data)


def _getHashTagRegex() -> Pattern:
    '''
        HashTag finder regular expression
    '''
    return regCompile(r'(\#\w+)')


def countOfHashTags(data: map) -> Counter:
    '''
        Returns count of each hashtags found in liked tweets
    '''
    _regex = _getHashTagRegex()

    return Counter(chain.from_iterable(map(lambda e: _regex.findall(e[1]),
                                           data)))


def topXHashTagsInLikedTweets(data: map, x: int) -> List[Tuple[str, int]]:
    '''
        Returns list of top X most used hash tags found in liked tweets
    '''
    return countOfHashTags(data).most_common(x)


def _getTaggedUserRegex() -> Pattern:
    '''
        Regular expression for finding tagged users
    '''
    return regCompile(r'(@\w+)')


def countOfTaggedUsers(data: map) -> Counter:
    '''
        Returns how many times each tagged user name found in liked tweets
    '''
    _regex = _getTaggedUserRegex()

    return Counter(chain.from_iterable(map(lambda e: _regex.findall(e[1]),
                                           data)))


def topXTaggedUsersInLikedTweets(data: map, x: int) -> List[Tuple[str, int]]:
    '''
        Returns list of top X mostly tagged usernames found in liked tweets
    '''
    return countOfTaggedUsers(data).most_common(x)


def _extractEmojis(text: str) -> List[str]:
    '''
        Given a text, extracts out all emojis present in that text
    '''
    return [i['emoji'] for i in emoji_lis(text)]


def countOfEmojis(data: map) -> Counter:
    '''
        Extracts out all emojis from liked tweets & finds their respective count
    '''
    return Counter(chain.from_iterable(
        map(lambda e: _extractEmojis(e[1]), data)))


def topXEmojisInLikedTweets(data: map, x: int) -> List[Tuple[str, int]]:
    '''
        Extracts top X most found emojis in liked tweets by you
    '''
    return countOfEmojis(data).most_common(x)


def getHashTagsAndEmojis(data: map) -> map:
    '''
        Extracts out hash tags and emojis present in each of tweets liked by YOU
    '''
    _regex = _getHashTagRegex()

    return map(lambda e: (_regex.findall(e[1]), _extractEmojis(e[1])), data)


def classifyEmojisByHashTagsInTweets(data: map) -> Dict[str, List[str]]:
    '''
        Given a map of tuples of this form `(['#100DaysOfCode'], ['👉', '♥', '😁'])`,
        we'll iterate over each hashtag & then append emojis to its own class i.e.
        classifying emojis by hashtags. 

        This association will help us in finding which emojis are used with which hashtags mostly,
        in liked tweets.
    '''
    _buffer = {}

    for i in getHashTagsAndEmojis(data):
        for j in i[0]:

            if j not in _buffer:
                _buffer[j] = [*i[1]]
            else:
                _buffer[j].extend(i[1])

    return _buffer


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

from typing import List, Dict, Any


def getAccountDisplayName(data: List[Dict[str, Any]]) -> str:
    return data[0]['account']['accountDisplayName']


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

from __future__ import annotations
from .engagement import Engagement
from .criteria import MatchedCriteria
from typing import List, Dict, Any, Tuple
from itertools import chain
from collections import Counter
from functools import reduce


class Engagements:
    def __init__(self, all: List[Engagement]):
        self.all = all

    @staticmethod
    def build(data: Dict[str, Any]) -> Engagements:
        return Engagements(list(
            chain.from_iterable(
                [[Engagement.build(j) for j in i['ad']['adsUserData']
                  ['adEngagements']['engagements']] for i in data])))

    def countByDeviceType(self) -> Counter:
        return Counter(filter(lambda e: e,
                              map(lambda e: e.device.type,
                                  self.all)))

    def countByDisplayLocation(self) -> Counter:
        '''
            Counts how many twitter ads were displayed on which location
            of your profile while you were browsing twitter
        '''
        return Counter(map(lambda e: e.displayLocation, self.all))

    def percentageOfAdsByDisplayLocation(self) -> List[Tuple[str, float]]:
        '''
            Percentage distribution of total ads shown on twitter to YOU,
            in terms of location of ad display
        '''
        _buffer = self.countByDisplayLocation()
        _total = sum(_buffer.values())

        return [(i, (_buffer[i] / _total) * 100) for i in sorted(_buffer,
                                                                 key=lambda e: _buffer[e],
                                                                 reverse=True)]

    def groupAdCountByDeviceTypeAndDisplayLocation(self) -> Dict[str, Counter]:
        '''
            Distribution of shown twitter ads by device type & respective location
            of display on device
        '''
        def _groupByDeviceType(acc: Dict[str, List[str]], cur: Tuple[str, str]) -> Dict[str, List[str]]:
            if cur[0] in acc:
                acc[cur[0]].append(cur[1])
            else:
                acc[cur[0]] = [cur[1]]

            return acc

        return dict(map(lambda e: (e[0], Counter(e[1])),
                        reduce(_groupByDeviceType,
                               map(lambda e: (e.device.type, e.displayLocation),
                                   self.all),
                               {}).items()))

    def adCountGroupedByAdvertiserName(self) -> Counter:
        '''
            How many ads this user engaged in from each advertiser
        '''
        return Counter(map(lambda e: e.advertiser.fullName, self.all))

    def topXAdvertiserNames(self, x: int) -> List[Tuple[str, int]]:
        '''
            Returns a list of top X advertisers in terms of number of ads 
            engaged in
        '''
        return self.adCountGroupedByAdvertiserName().most_common(x)

    def adCountGroupedByEngagementType(self) -> Counter:
        '''
            Counts how many ad engagements happened in which way i.e.
            {EmbeddedMedia, ChargeableImpression}
        '''
        return Counter(chain.from_iterable(
            map(lambda e: map(lambda _e: _e.type, e.attrs), self.all)))

    def topXEngagementTypes(self, x: int) -> List[Tuple[str, int]]:
        '''
            Top X ways how you got engaged with a twitter advertisement i.e.
            rather twitter engaged you in
        '''
        return self.adCountGroupedByEngagementType().most_common(x)

    def groupAdCountByAdvertiserNameAndEngagementTypes(self):
        '''
            Groups advertisement counts by advertiser name & under each of those
            respective engagement type used for showing advertisement, along with their
            count
        '''
        def _groupIt(acc: Dict[str, List[map]], cur: Tuple[str, map]) -> Dict[str, List[map]]:
            if cur[0] in acc:
                acc[cur[0]].append(cur[1])
            else:
                acc[cur[0]] = [cur[1]]

            return acc

        return dict(map(lambda e: (e[0], Counter(chain.from_iterable(e[1]))),
                        reduce(_groupIt, map(lambda e: (e.advertiser.fullName,
                                                        map(lambda _e: _e.type, e.attrs)), self.all), {}).items()))

    def topXAdvertiserNamesWithRespectiveEngagementTypes(self, x: int):
        '''
            Returns top X advertisers with their respective engagement types & counts used
            for showing you ad on twitter
        '''
        _ads = self.groupAdCountByAdvertiserNameAndEngagementTypes()

        _topX = sorted(_ads,
                       key=lambda e: sum(_ads[e].values()),
                       reverse=True)[:x]

        return dict(map(lambda e: (e, _ads[e]), _topX))

    def groupEngagementsByAdvertiserName(self) -> Dict[str, List[Engagement]]:
        '''
            Groups all engagements by their respective advertiser's screen name
        '''
        def _groupBy(acc: Dict[str, List[Engagement]], cur: Engagement) -> Dict[str, List[Engagement]]:
            if cur.advertiser.fullName in acc:
                acc[cur.advertiser.fullName].append(cur)
            else:
                acc[cur.advertiser.fullName] = [cur]

            return acc

        return reduce(_groupBy, self.all, {})

    def getEngagementsByAdvertiserName(self, advertiser: str) -> List[Engagement]:
        '''
            Returns a list of all engagements, when advertiser name is given
        '''
        return self.groupEngagementsByAdvertiserName().get(advertiser)

    def getMatchedTargetingCriteriasByAdvertiserName(self, advertiser: str) -> List[MatchedCriteria]:
        '''
            Returns a list of all advertise targeting criterias used by a certain advertiser,
            along with their respective values set by them
        '''
        return list(chain.from_iterable(map(lambda e: map(lambda _e: _e, e.criterias),
                                            self.getEngagementsByAdvertiserName(advertiser))))

    def getGroupedTargetingCriteriasByAdvertiserName(self, advertiser: str) -> Dict[str, Counter]:
        '''
            Groups all targeting criterias for a specific advertiser, by their
            respective criteria type & under each of them keep a counter for all values
        '''
        def _groupBy(acc: Dict[str, List[str]], cur: MatchedCriteria) -> Dict[str, List[str]]:
            if cur.type in acc:
                acc[cur.type].append(cur.value)
            else:
                acc[cur.type] = [cur.value]

            return acc

        return dict(map(lambda e: (e[0], Counter(e[1])),
                        reduce(_groupBy,
                               self.getMatchedTargetingCriteriasByAdvertiserName(
                                   advertiser),
                               {}).items()))

    def usageCountByAdTargetCriteriaValue(self) -> Counter:
        '''
            Counting number of occurance of ad target criteria values
        '''
        return Counter(chain.from_iterable(
            map(lambda e: map(lambda _e: _e.value,
                              e.criterias), self.all)))

    def topXAdTargetCriteriasUsed(self, x: int) -> List[Dict[str, int]]:
        '''
            Finding top X ad target criteria values
        '''
        return self.usageCountByAdTargetCriteriaValue().most_common(x)

    def adTargetCriteriaUsageCountGroupedByType(self) -> Dict[str, Counter]:
        '''
            Grouping ad target criteria usage count by targeting criteria type 
            & under each type, holding the usage count of each target value
        '''
        def _groupBy(acc: Dict[str, List[str]], cur: Tuple[str, str]) -> Dict[str, List[str]]:
            if cur[0] in acc:
                acc[cur[0]].append(cur[1])
            else:
                acc[cur[0]] = [cur[1]]

            return acc

        return dict(map(lambda e: (e[0], Counter(e[1])),
                        reduce(_groupBy,
                               chain.from_iterable(map(lambda e: map(lambda _e: (_e.type, _e.value),
                                                                     e.criterias), self.all)), {}).items()))

    def hashTagCountInPromotedTweets(self) -> Counter:
        '''
            Finds all hashtags used in all promoted tweets you got engaged in,
            with their respective count of usage
        '''
        return Counter(chain.from_iterable(
            map(lambda e: e.extractHashTags(),
                filter(lambda e: not e.isEmpty,
                       map(lambda e: e.tweet, self.all)))))

    def topXHashTagsUsedInPromotedTweets(self, x: int) -> List[Tuple[str, int]]:
        '''
            Returns a list of most common X hash tags found in promoted tweets
        '''
        return self.hashTagCountInPromotedTweets().most_common(x)

    def hashTagCountInPromotedTweetsGroupedByAdvertisers(self) -> Dict[str, Counter]:
        '''
            Grouping all hashtags found in promoted tweets by their respective advertiser name
            and keeping count of each hashtag's occurance
        '''
        def _groupBy(acc: Dict[str, List[List[str]]], cur: Tuple[str, List[str]]) -> Dict[str, List[List[str]]]:
            if not cur[1]:
                return acc

            if cur[0] in acc:
                acc[cur[0]].append(cur[1])
            else:
                acc[cur[0]] = [cur[1]]

            return acc

        return dict(map(
            lambda e: (e[0], Counter(chain.from_iterable(e[1]))),
            reduce(_groupBy,
                   map(lambda e: (e[0], e[1].extractHashTags()),
                       filter(lambda e: not e[1].isEmpty,
                              map(lambda e: (e.advertiser.fullName, e.tweet),
                                  self.all))), {}).items()))

    def topXHashTagsUsedByTopYAdvertisersInPromotedTweets(self, x: int, y: int) -> map:
        '''
            Finding top X used hash tags in promoted tweets by top Y advertisers 
            ( where top Y determined number of tweets they promoted )
        '''
        _hashtags = self.hashTagCountInPromotedTweetsGroupedByAdvertisers()

        return map(lambda e: (e, _hashtags[e].most_common(x)),
                   sorted(_hashtags,
                          key=lambda e: sum(_hashtags[e].values()),
                          reverse=True)[:y])

    def taggedAccountCountInPromotedTweets(self) -> Counter:
        '''
            Extracts out all tagged twitter user names found in
            promoted tweets, along with their respective occurance count
        '''
        return Counter(chain.from_iterable(
            map(lambda e: e.extractTags(),
                filter(lambda e: not e.isEmpty,
                       map(lambda e: e.tweet, self.all)))))

    def topXTaggedAccountsInPromotedTweets(self, x: int) -> List[Tuple[str, int]]:
        '''
            Given x > 0, it'll find out top X most tagged twitter user names, found in promoted tweets 
        '''
        return self.taggedAccountCountInPromotedTweets().most_common(x)


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

from dataclasses import dataclass
from datetime import datetime


@dataclass
class EngagementAttributes:
    _time: str
    type: str

    def time(self) -> datetime:
        return datetime.strptime(self._time, '%Y-%m-%d %H:%M:%S')


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

from typing import Set, List


def getFollowedFollowers(followers: map, followings: map) -> Set[str]:
    '''
        Computes list of those followings who follow you back or in other terms
        followers whom you follow back
    '''
    return set(map(lambda e: e[0], followers)).intersection(
        set(map(lambda e: e[0], followings)))


def getFollowersAndFollowingsPerCent(followers: map, followings: map) -> List[float]:
    '''
        Scaling down number of followers & followings to 100 i.e. per cent of each of these categories
    '''
    _tmp = list(map(lambda e: len(list(e)), [followers, followings]))

    return [(i / sum(_tmp)) * 100 for i in _tmp]


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

from re import compile as regCompile, Pattern
from typing import List, Dict, Any
from os.path import exists
from json import loads, JSONDecodeError


def _getRegex() -> Pattern:
    '''
        Compile regular expression to be for parsing out data from *.js file
    '''
    return regCompile(r'(=([\s|\S]+))$')


def parse(src: str) -> List[Dict[str, Any]]:
    '''
        Parses JS file content & returns Python object
    '''
    if not exists(src):
        return None

    _reg = _getRegex()
    with open(src, mode='r') as fd:
        data = fd.read()

    _match = _reg.search(data)
    if not _match:
        return None

    try:
        return loads(_match.group(2).strip())
    except JSONDecodeError:
        return None


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

from __future__ import annotations
from .device import DeviceInfo
from .tweet import Tweet
from .advertiser import Info
from .criteria import MatchedCriteria
from .attributes import EngagementAttributes
from datetime import datetime
from typing import List, Dict, Any


class Engagement:
    def __init__(self, device: DeviceInfo, location: str, tweet: Tweet,
                 advertiser: Info, criterias: List[MatchedCriteria],
                 time: str, attrs: List[EngagementAttributes]):
        self.device = device
        self.displayLocation = location
        self.tweet = tweet
        self.advertiser = advertiser
        self.criterias = criterias
        self.time = datetime.strptime(time, '%Y-%m-%d %H:%M:%S')
        self.attrs = attrs

    @staticmethod
    def build(data: Dict[str, Any]) -> Engagement:
        impressionAttr = data['impressionAttributes']

        return Engagement(
            DeviceInfo(impressionAttr['deviceInfo'].get('osType'),
                       impressionAttr['deviceInfo'].get('deviceId')),
            impressionAttr.get('displayLocation'),
            Tweet(
                impressionAttr.get('promotedTweetInfo', {}).get('tweetId'),
                impressionAttr.get('promotedTweetInfo', {}).get('tweetText'),
                impressionAttr.get('promotedTweetInfo', {}).get('urls'),
                impressionAttr.get('promotedTweetInfo', {}).get('mediaUrls')
            ),
            Info(
                impressionAttr['advertiserInfo']['advertiserName'],
                impressionAttr['advertiserInfo']['screenName']
            ),
            [MatchedCriteria(i.get('targetingType'), i.get('targetingValue'))
             for i in impressionAttr.get('matchedTargetingCriteria', [])],
            impressionAttr['impressionTime'],
            [EngagementAttributes(i['engagementTime'], i['engagementType'])
             for i in data['engagementAttributes']])


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

import seaborn as sns
from matplotlib import pyplot as plt
from typing import List, Tuple, Any
from twiz.model.ad.engagement.engagements import Engagements
from functools import reduce
from itertools import chain
from collections import Counter


def plotAdTargetDeviceTypes(data: Engagements, title: str, sink: str) -> bool:
    try:
        _tmp = data.countByDeviceType()
        x = list(_tmp.keys())
        y = [_tmp[i] for i in x]

        fig = plt.Figure(figsize=(16, 9), dpi=100)
        sns.set_style('darkgrid')

        sns.barplot(x=x, y=y, ax=fig.gca())

        for i, j in enumerate(fig.gca().patches):
            fig.gca().text(j.get_x() + j.get_width() * .5,
                           j.get_y() + j.get_height() * .5,
                           y[i],
                           ha='center',
                           rotation=0,
                           fontsize=14,
                           color='black')

        fig.gca().set_title(title, fontsize=20, pad=10)
        fig.tight_layout(pad=4)

        fig.savefig(sink, pad_inches=.8)
        plt.close(fig)

        return True
    except Exception:
        return False


def plotPercentageOfShownAdsByLocationOfDisplay(data: Engagements, title: str, sink: str) -> bool:
    try:
        _tmp = data.percentageOfAdsByDisplayLocation()
        x = [i[0] for i in _tmp]
        y = [i[1] for i in _tmp]

        _counts = data.countByDisplayLocation()
        _counts = [_counts[i] for i in x]

        with plt.style.context('dark_background'):
            fig = plt.Figure(figsize=(16, 9), dpi=100)

            sns.barplot(x=x, y=y, ax=fig.gca(), palette='BuGn_r')

            for i, j in enumerate(fig.gca().patches):
                fig.gca().text(j.get_x() + j.get_width() * .5,
                               j.get_y() + j.get_height() * .5,
                               _counts[i],
                               ha='center',
                               rotation=0,
                               fontsize=16,
                               color='white')

            fig.gca().set_ylim(0, 100)
            fig.gca().set_title(title, fontsize=20, pad=14)
            fig.tight_layout(pad=4)

            fig.savefig(sink, pad_inches=.8)
            plt.close(fig)

        return True
    except Exception:
        return False


def _prepareDataForTwitterAdsGroupedByDeviceTypeAndDisplayLocation(data: Engagements) -> Tuple[List[str], List[int], List[str]]:
    '''
        Preparing data for plotting group bar chart, depicting how many twitter ads
        were shown on which device & how many in each of them were shown on which portion
        of display
    '''
    _tmp = data.groupAdCountByDeviceTypeAndDisplayLocation()
    _displayLocations = list(set(reduce(
        lambda acc, cur: acc + list(cur),
        map(lambda e: e.keys(), _tmp.values()), [])))

    _x = list(chain.from_iterable(
        [[i] * len(_displayLocations) for i in _tmp.keys()]))
    _y = [_tmp[j].get(_displayLocations[i % len(_displayLocations)], 0)
          for i, j in enumerate(_x)]
    _hue = list(chain.from_iterable(
        [_displayLocations * (len(_x) // len(_displayLocations))]))

    return _x, _y, _hue


def plotShownAdsGroupedByDeviceTypeAndDisplayLocation(data: Engagements, title: str, sink: str) -> bool:
    '''
        Plotting grouped bar chart, showing how many twitter ads
        were shown on which device & how many in each of them 
        were shown on which portion of display
    '''
    try:
        _x, _y, _hue = _prepareDataForTwitterAdsGroupedByDeviceTypeAndDisplayLocation(
            data)

        with plt.style.context('dark_background'):
            fig = plt.Figure(figsize=(16, 9), dpi=100)

            sns.barplot(x=_x, y=_y, hue=_hue,
                        ax=fig.gca(), palette='PuRd')

            fig.gca().set_title(title, fontsize=20, pad=14)
            fig.tight_layout(pad=4)

            fig.savefig(sink, pad_inches=.8)
            plt.close(fig)

        return True
    except Exception:
        return False


def plotAdsCountGroupedByAdvertiserNames(data: Engagements, x: int, title: str, sink: str) -> bool:
    '''
        We're going to plot how many ads user has engaged in from certain advertiser,
        as a bar plot. Top X advertisers to be plotted, denoted by their name, userid omitted.
    '''
    try:
        _ads = data.topXAdvertiserNames(x)

        _x = [i[1] for i in _ads]
        _y = [i[0] for i in _ads]

        with plt.style.context('dark_background'):
            fig = plt.Figure(figsize=(16, 9), dpi=100)

            sns.barplot(x=_x, y=_y, ax=fig.gca(),
                        palette='PuRd', orient='h')

            fig.gca().set_title(title, fontsize=20, pad=16)
            fig.tight_layout(pad=4)

            fig.savefig(sink, pad_inches=.8)
            plt.close(fig)

        return True
    except Exception:
        return False


def plotAdsCountGroupedByEngagementTypes(data: Engagements, x: int, title: str, sink: str) -> bool:
    '''
        Plotting ad count by grouping ads by their engagement types i.e.
        depicts how twitter ads were trying to engage you & in which way they happened
        mostly.
    '''
    try:
        _ads = data.topXEngagementTypes(x)

        _x = [i[1] for i in _ads]
        _sum = sum(_x)
        _y = [i[0] for i in _ads]

        with plt.style.context('dark_background'):
            fig = plt.Figure(figsize=(16, 9), dpi=100)

            sns.barplot(x=_x, y=_y, ax=fig.gca(),
                        palette='YlOrRd', orient='h')

            for i, j in enumerate(fig.gca().patches):
                fig.gca().text(j.get_x() + j.get_width() * .5,
                               j.get_y() + j.get_height() * .5,
                               '{:.2f} %'.format((_x[i] / _sum) * 100),
                               ha='center',
                               rotation=0,
                               fontsize=10,
                               color='black')

            fig.gca().set_title(title, fontsize=20, pad=16)
            fig.tight_layout()

            fig.savefig(sink, pad_inches=.8)
            plt.close(fig)

        return True
    except Exception:
        return False


def _prepareDataForTopXAdvertisersWithRespectiveEngagementTypes(data: Engagements, x: int) -> Tuple[List[str], List[str], List[List[str]]]:
    '''
        Preparing data for plotting heat map showing which of top advertisers used
        what kind of engagement types how many times. 
    '''
    _ads = data.topXAdvertiserNamesWithRespectiveEngagementTypes(x)

    _x = sorted(set(chain.from_iterable(
        [tuple(i.keys()) for i in _ads.values()])))
    _y = sorted(_ads, key=lambda e: sum(_ads[e].values()))

    _counts = [[_ads[i].get(j, 0) for j in _x] for i in _y]

    return _y, _x, _counts


def plotTopXAdvertisersAsHeatMap(data: Engagements, x: int, title: str, sink: str) -> bool:
    '''
        Plotting top X advertisers with their respective engagement type & count
        as heatmap
    '''
    try:
        _y, _x, _data =\
            _prepareDataForTopXAdvertisersWithRespectiveEngagementTypes(
                data, x)

        with plt.style.context('dark_background'):
            fig = plt.Figure(figsize=(18, 10), dpi=200)

            sns.heatmap(_data, lw=.75,
                        cmap='YlGnBu', ax=fig.gca())

            fig.gca().set_xticklabels(_x, rotation=72)
            fig.gca().tick_params(axis='x', which='major', labelsize=10)
            fig.gca().set_yticklabels(_y, rotation=0)
            fig.gca().tick_params(axis='y', which='major', labelsize=12)

            fig.gca().set_title(title, fontsize=20, pad=16)
            fig.tight_layout()

            fig.savefig(sink, pad_inches=.8)
            plt.close(fig)

        return True
    except Exception:
        return False


def _prepareDataForPlottingTargetCriteriasByAdvertiserName(data: Engagements, advertiser: str) -> map:
    def _prepareDataForTargetType(elem: str) -> Tuple[str, List[str], List[int]]:
        _x = sorted(_data[elem].keys(), key=lambda e: _data[elem][e])
        _y = [_data[elem][i] for i in _x]

        return elem, _x, _y

    _data = data.getGroupedTargetingCriteriasByAdvertiserName(advertiser)
    _keys = list(filter(lambda e: len(_data[e]) > 1, _data.keys()))

    return map(_prepareDataForTargetType, _keys)


def _joinName(name: str) -> str:
    return '_'.join(name.split())


def plotTargetCriteriasForTopXAdvertisers(data: Engagements, x: int, title: str, sink: str) -> bool:
    try:
        def _plot(advertiser: str, type: str, _x: List[str], _y: List[float]):
            with plt.style.context('dark_background'):
                fig = plt.Figure(figsize=(16, 9), dpi=100)

                sns.barplot(x=_y, y=_x, ax=fig.gca(),
                            palette='BuPu', orient='h')

                fig.gca().set_title('{}{} under {} category'.format(title, advertiser, type),
                                    fontsize=20, pad=16)
                fig.tight_layout()

                fig.savefig('{}{}{}.png'.format(sink, _joinName(advertiser), _joinName(type)),
                            pad_inches=.8)
                plt.close(fig)

        for k in data.topXAdvertiserNames(x):
            for v in _prepareDataForPlottingTargetCriteriasByAdvertiserName(data, k[0]):
                _plot(k[0], *v)

        return True
    except Exception:
        return False


def plotTopXTargetCriteriasUsedByTwitterAdvertisers(data: Engagements, x: int, title: str, sink: str) -> bool:
    '''
        Plotting a bar chart depicting which ad target criteria was used mostly for targeting you
    '''
    try:
        _criterias = data.topXAdTargetCriteriasUsed(x)

        _x = [i[1] for i in _criterias]
        _y = [i[0] for i in _criterias]

        with plt.style.context('dark_background'):
            fig = plt.Figure(figsize=(16, 9), dpi=200)

            sns.barplot(x=_x, y=_y,
                        ax=fig.gca(),
                        palette='RdPu',
                        orient='h')

            fig.gca().set_title(title, fontsize=20, pad=16)
            fig.tight_layout()

            fig.savefig(sink, pad_inches=.8)
            plt.close(fig)

        return True
    except Exception:
        return False


def _prepareDataForPlottingAdTargetCriteriaUsageGroupedByType(data: Engagements) -> map:
    '''
        Splitting ad target criteria usage count into X -axis data & Y -axis data,
        then creating a map _( lazy evaluation )_, where each element is a
        tuple of ad target criteria type & respective X, Y -axis value to be plotted.
    '''
    def _extractXAndY(v: Counter) -> Tuple[List[str], List[str]]:
        _x = list(v.keys())[:30]
        _y = [v[i] for i in _x]

        return _x, _y

    return map(lambda e: (e[0], *_extractXAndY(e[1])),
               data.adTargetCriteriaUsageCountGroupedByType().items())


def plotBarChartShowingAdTargetCriteriaUsageCountGroupedByType(data: Engagements, name: str):
    '''
        Plotting Twitter Ad target criterias used under each category 
        i.e. {Locations, Age, Follower look-alikes, Platforms, ...}, as bar plot
    '''
    def _plot(title: str, sink: str, _x: List[str], _y: List[str]):
        with plt.style.context('dark_background'):
            fig = plt.Figure(figsize=(24, 12), dpi=240)

            sns.barplot(x=_y, y=_x,
                        ax=fig.gca(),
                        palette='plasma',
                        orient='h')

            fig.gca().set_title(title, fontsize=18, pad=10)
            fig.tight_layout()

            fig.savefig(sink, pad_inches=.8)
            plt.close(fig)

    try:
        for i in _prepareDataForPlottingAdTargetCriteriaUsageGroupedByType(data):
            _plot('Using `{}` as Twitter Ad Target Criteria, for {}'.format(i[0], name),
                  'plots/twitterAdTargetCriteriasUsedIn{}For{}.png'.format(
                      _joinName(i[0]), _joinName(name)),
                  *i[1:])

        return True
    except Exception:
        return False


def plotTopXHashTagsInPromotedTweets(data: Engagements, x: int, title: str, sink: str) -> bool:
    '''
        Plotting most common X number hash tags found in promoted tweets,
        as horizontal bar plot
    '''
    try:
        _hashtags = data.topXHashTagsUsedInPromotedTweets(x)

        _x = [i[0] for i in _hashtags]
        _y = [i[1] for i in _hashtags]

        with plt.style.context('dark_background'):
            fig = plt.Figure(figsize=(16, 9), dpi=200)

            sns.barplot(x=_y, y=_x,
                        ax=fig.gca(),
                        palette='viridis',
                        orient='h')

            fig.gca().set_title(title, fontsize=18, pad=10)
            fig.tight_layout()

            fig.savefig(sink, pad_inches=.8)
            plt.close(fig)

        return True
    except Exception:
        return False


def plotTopXHashTagsUsedByTopYAdvertisersInPromotedTweets(data: Engagements, x: int, y: int, title: str, sink: str) -> bool:
    '''
        Plotting grouped bar chart depicting top X hash tags used by top Y advertiser's
        promoted tweets.
    '''
    def _prepareData() -> Tuple[List[str], List[int], List[str], List[str]]:
        '''
            Prepare data for plotting grouped bar chart
        '''
        def _handleAbsenceOfXHashTags(_tmp: List[List[Any]], _v: Any) -> List[List[Any]]:
            '''
                If some advertisers are not having X elements, then we're going to
                fill empty spaces i.e. X - len(cur_element_set), with default value `_v`
            '''
            return reduce(lambda acc, cur: acc +
                          [cur + [_v] * (x - len(cur))], _tmp, [])

        _hashTags = list(
            data.topXHashTagsUsedByTopYAdvertisersInPromotedTweets(x, y))

        _x = list(chain.from_iterable([[i[0]] * x for i in _hashTags]))

        _y = list(chain.from_iterable(
            _handleAbsenceOfXHashTags(
                [[j[1] for j in i[1]] for i in _hashTags], 0)))
        _labels = list(chain.from_iterable(
            _handleAbsenceOfXHashTags(
                [[j[0] for j in i[1]] for i in _hashTags], '')))

        _hues = [f'#HashTag : {i+1}' for i in range(x)] * y

        return _x, _y, _labels, _hues

    def _splitAndZip(_tmp: List[str]):
        '''
            Split an iterator into collection of X -sized subsets, then zip them,
            such that following happens.

            _l = [1, 2, 3, 4, 5, 6]
            _splitted = [[1, 2, 3], [4, 5, 6]] ( into 3 -sized subsets )
            _zipped = [(1, 4), (2, 5), (3, 6)]
            _joined = [1, 4, 2, 5, 3, 6] -> this is to be used as label names
        '''
        return list(chain.from_iterable(
            zip(*[_tmp[i:i+x] for i in range(0, len(_tmp), x)])))

    try:
        _x, _y, _labels, _hues = _prepareData()

        with plt.style.context('dark_background'):
            fig = plt.Figure(figsize=(20, 12), dpi=200)

            sns.barplot(x=_y, y=_x, hue=_hues,
                        ax=fig.gca(),
                        palette='Blues_d',
                        orient='h')

            _splittedAndZipped = _splitAndZip(_labels)

            for i, j in enumerate(fig.gca().patches):
                fig.gca().text(j.get_x() + j.get_width() * .5,
                               j.get_y() + j.get_height() * .5,
                               _splittedAndZipped[i],
                               ha='center',
                               rotation=0,
                               fontsize=9,
                               color='black')

            fig.gca().set_title(title, fontsize=18, pad=10)
            fig.tight_layout(pad=4)

            fig.savefig(sink, pad_inches=.8)
            plt.close(fig)

        return True
    except Exception:
        return False

def plotTopXTaggedAccountsInPromotedTweets(data: Engagements, x: int, title: str, sink: str) -> bool:
    '''
        Plotting most commonly found X number tagged usernames, in promoted tweets,
        as horizontal bar plot
    '''
    try:
        _hashtags = data.topXTaggedAccountsInPromotedTweets(x)

        _x = [i[0] for i in _hashtags]
        _y = [i[1] for i in _hashtags]

        with plt.style.context('dark_background'):
            fig = plt.Figure(figsize=(16, 9), dpi=200)

            sns.barplot(x=_y, y=_x,
                        ax=fig.gca(),
                        palette='cividis',
                        orient='h')

            fig.gca().set_title(title, fontsize=18, pad=10)
            fig.tight_layout()

            fig.savefig(sink, pad_inches=.8)
            plt.close(fig)

        return True
    except Exception:
        return False

if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

from typing import List, Dict, Any


def getFollowers(data: List[Dict[str, Any]]) -> map:
    '''
        Obtains a list of followers, where each of them will be a
        tuple of accountId & corresponding user's account link
    '''
    return map(lambda e: (e['follower']['accountId'],
                          e['follower']['userLink']), data)


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

from os import mkdir
from os.path import exists
from zipfile import ZipFile, BadZipFile


def makeDirectory(target: str) -> bool:
    '''
        Creates non-existing directory & denotes boolean status
    '''
    if exists(target):
        return True

    try:
        mkdir(target)

        return True
    except OSError:
        return False


def extract(src: str, sink: str) -> bool:
    '''
        Extracts compressed twitter account data file into sink directory
    '''
    if not (exists(src) and src.endswith('.zip') and makeDirectory(sink)):
        return False

    try:
        _zf = ZipFile(src)
        _zf.extractall(sink)

        return True
    except BadZipFile:
        return False


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

from typing import List
from re import compile as regCompile, I as regI, Pattern


class Tweet:
    def __init__(self, id: str, text: str, urls: List[str], mediaUrls: List[str]):
        self.id = id
        self.text = text
        self.urls = urls
        self.mediaUrls = mediaUrls

    def _tagRegex(self) -> Pattern:
        return regCompile(r'(@\w+)')

    def extractTags(self) -> List[str]:
        return self._tagRegex().findall(self.text)

    def _hashTagRegex(self) -> Pattern:
        return regCompile(r'(\#\w+)')

    def extractHashTags(self) -> List[str]:
        return self._hashTagRegex().findall(self.text)

    @property
    def isEmpty(self) -> bool:
        '''
            Checks whether promoted tweet info is present or not
        '''
        return self.text is None


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')



#!/usr/bin/python3

from argparse import ArgumentParser
from typing import Tuple, List
from os.path import exists, abspath, join
from .extract import extract, makeDirectory
from .parser import parse
from .model.follower import getFollowers
from .model.following import getFollowings
from .model.account import getAccountDisplayName
from .model.like import getLikes
from .model.ad.engagement.engagements import Engagements
from .plot.util import (
    plotFollowersAndFollowings,
    plotFollowersAndFollowedFollowers,
    plotTopXHashTagsFoundInLikedTweets,
    plotTopXTaggedUsersFoundInLikedTweets,
    plotTopXEmojisFoundInLikedTweets
)
from .plot.ad.engagement.viz import (
    plotAdTargetDeviceTypes,
    plotPercentageOfShownAdsByLocationOfDisplay,
    plotShownAdsGroupedByDeviceTypeAndDisplayLocation,
    plotAdsCountGroupedByAdvertiserNames,
    plotAdsCountGroupedByEngagementTypes,
    plotTopXAdvertisersAsHeatMap,
    plotTargetCriteriasForTopXAdvertisers,
    plotTopXTargetCriteriasUsedByTwitterAdvertisers,
    plotBarChartShowingAdTargetCriteriaUsageCountGroupedByType,
    plotTopXHashTagsInPromotedTweets,
    plotTopXHashTagsUsedByTopYAdvertisersInPromotedTweets,
    plotTopXTaggedAccountsInPromotedTweets
)
from time import time


def _getCMD() -> Tuple[str, str]:
    parser = ArgumentParser(
        prog='twiz',
        description='twiz - Your Twitter Account Data Analysis & Visualization Tool <3')
    parser.add_argument('src', type=str, nargs=1,
                        help='Compressed File with Twitter Account Data')
    parser.add_argument('sink', type=str, nargs=1,
                        help='Zip extraction directory')
    args = parser.parse_args()

    return (abspath(args.src[0]), abspath(args.sink[0]))


def _calculateSuccess(_data: List[bool]) -> float:
    return (len(list(filter(lambda e: e, _data))) / len(_data)) * 100


def _banner():
    print('\x1b[1;6;36;49m[+]twiz v0.3.0 - Your Twitter Account Data Analysis & Visualization Tool <3\x1b[0m\n\n\t\x1b[3;39;40m$ twiz `path-to-zip-file` `path-to-sink-directory`\x1b[0m\n\n[+]Author: Anjan Roy <anjanroy@yandex.com>\n[+]Source: https://github.com/itzmeanjan/twiz ( CC0-1.0 Licensed )\n')


def _joinName(name: str) -> str:
    return '_'.join(name.split())


def main():
    _banner()

    try:
        src, sink = _getCMD()
        if not exists(src):
            raise RuntimeError('Source file absent')

        if not extract(src, sink):
            raise RuntimeError('Failed to extract from source')

        print('[+]Working ...')
        _start = time()

        _parsedFollowers = parse(join(sink, 'data/follower.js'))
        if not _parsedFollowers:
            raise RuntimeError('Failed to parse followers')

        _parsedFollowings = parse(join(sink, 'data/following.js'))
        if not _parsedFollowings:
            raise RuntimeError('Failed to parse followings')

        _account = parse(join(sink, 'data/account.js'))
        if not _account:
            raise RuntimeError('Failed to parse account')

        _parsedLikes = parse(join(sink, 'data/like.js'))
        if not _parsedLikes:
            raise RuntimeError('Failed to parse likes')

        _parsedAdEngagement = parse(join(sink, 'data/ad-engagements.js'))
        if not _parsedFollowers:
            raise RuntimeError('Failed to parse advertisement engagements')

        _followers = getFollowers(_parsedFollowers)
        _followings = getFollowings(_parsedFollowings)
        _accountDisplayName = getAccountDisplayName(_account)
        _likes = getLikes(_parsedLikes)
        _engagements = Engagements.build(_parsedAdEngagement)
        makeDirectory('plots')

        _success = [
            plotFollowersAndFollowings(_followers,
                                       _followings,
                                       'Twitter Followers And Followings for {}'
                                       .format(_accountDisplayName),
                                       'plots/twitterFollowersAndFollowingsFor{}.png'
                                       .format(_joinName(_accountDisplayName))),
            plotFollowersAndFollowedFollowers(_followers,
                                              _followings,
                                              ['Twitter Followers & Followed back Followers for {}'.format(_accountDisplayName),
                                               'Twitter Followings & Follower Followings for {}'.format(_accountDisplayName)],
                                              'plots/twitterFollowersFollowingsAndIntersectionFor{}.png'
                                              .format(_joinName(_accountDisplayName))),
            plotTopXHashTagsFoundInLikedTweets(
                _likes, 10,
                'Top 10 Twitter #HASHTAGS found in tweets liked by {}'.format(
                    _accountDisplayName),
                'plots/top10TwitterHashTagsFoundInTweetsLikedBy{}.png'.format(
                    _joinName(_accountDisplayName))),
            plotTopXTaggedUsersFoundInLikedTweets(
                _likes, 10,
                'Top 10 @TaggedTwitterUsers found in tweets liked by {}'.format(
                    _accountDisplayName),
                'plots/top10TaggedTwitterUsersFoundInTweetsLikedBy{}.png'.format(
                    _joinName(_accountDisplayName))),
            plotTopXEmojisFoundInLikedTweets(
                _likes, 10,
                'Top 10 Emojis found in tweets liked by {}'.format(
                    _accountDisplayName),
                'plots/top10EmojisFoundInTweetsLikedBy{}.png'.format(
                    _joinName(_accountDisplayName))),
            plotAdTargetDeviceTypes(
                _engagements,
                'Twitter Ads for {}, by target device type'.format(
                    _accountDisplayName),
                'plots/twitterAdsTargeting{}OnDevices.png'.format(
                    _joinName(_accountDisplayName))),
            plotPercentageOfShownAdsByLocationOfDisplay(
                _engagements,
                'Twitter Ads for {}, by on-screen display location'.format(
                    _accountDisplayName),
                'plots/twitterAdCountByDisplayLocationFor{}.png'.format(
                    _joinName(_accountDisplayName))),
            plotShownAdsGroupedByDeviceTypeAndDisplayLocation(
                _engagements,
                'Twitter Ads for {}, grouped by Device Type & on-screen Display Location'.format(
                    _accountDisplayName),
                'plots/twitterAdsGroupedByDeviceTypeAndDisplayLocationFor{}.png'.format(
                    _joinName(_accountDisplayName))),
            plotAdsCountGroupedByAdvertiserNames(
                _engagements,
                15,
                f'Top 15 Twitter Advertisers {_accountDisplayName} interacted with',
                'plots/twitterAdsCountGroupedByAdvertiserNamesFor{}.png'.format(
                    _joinName(_accountDisplayName))),
            plotAdsCountGroupedByEngagementTypes(
                _engagements,
                15,
                f'Top 15 Ad Engagement Types Twitter used for {_accountDisplayName}',
                f'plots/twitterAdsCountGroupedByEngagementTypesFor{_joinName(_accountDisplayName)}.png'),
            plotTopXAdvertisersAsHeatMap(
                _engagements,
                15,
                f'Top 15 Twitter Advertisers with respective Engagement Types for {_accountDisplayName}',
                f'plots/twitterAdsCountGroupedByAdvertiserNamesAndEngagementTypesFor{_joinName(_accountDisplayName)}.png'),
            plotTargetCriteriasForTopXAdvertisers(
                _engagements,
                5,
                f'Twitter Ad Targeting Criterias used for {_accountDisplayName} by ',
                f'plots/twitterAdTargetingCriteriasUsedFor{_joinName(_accountDisplayName)}By'),
            plotTopXTargetCriteriasUsedByTwitterAdvertisers(
                _engagements,
                20,
                f'Top 20 Ad Target Criterias used for {_accountDisplayName} on Twitter',
                f'plots/top20AdTargetCriteriasUsedFor{_joinName(_accountDisplayName)}OnTwitter.png'),
            plotBarChartShowingAdTargetCriteriaUsageCountGroupedByType(
                _engagements,
                _accountDisplayName),
            plotTopXHashTagsInPromotedTweets(
                _engagements,
                20,
                f'Top 20 #HashTags in Promoted Tweets targeting {_accountDisplayName}',
                f'plots/top20HashTagsInPromotedTweetsFor{_joinName(_accountDisplayName)}.png'),
            plotTopXHashTagsUsedByTopYAdvertisersInPromotedTweets(
                _engagements, 3, 10,
                f'Top 3 #HashTags in Tweets promoted by top 10 Advertisers',
                f'plots/top3HashTagsInPromotedTweetsFromTop10AdvertisersFor{_joinName(_accountDisplayName)}.png'),
            plotTopXTaggedAccountsInPromotedTweets(
                _engagements,
                20,
                f'Top 20 @TaggedUsers in Promoted Tweets targeting {_accountDisplayName}',
                f'plots/top20TaggedUsersInPromotedTweetsFor{_joinName(_accountDisplayName)}.png')
        ]

        print('[+]Obtained success : {:.2f} %, in {:.2f} s'.format(
            _calculateSuccess(_success),
            time() - _start))
    except KeyboardInterrupt:
        print('\n[!]Terminated')
    except Exception as e:
        print('[!]Error: {}'.format(e))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['twiz',
 'twiz.model',
 'twiz.model.ad',
 'twiz.model.ad.engagement',
 'twiz.plot',
 'twiz.plot.ad',
 'twiz.plot.ad.engagement']

package_data = \
{'': ['*']}

install_requires = \
['matplotlib', 'seaborn', 'emoji']

extras_require = \
{'dev': ['flit']}

entry_points = \
{'console_scripts': ['twiz = twiz.main:main']}

setup(name='twiz',
      version='0.3.0',
      description='twiz - Your Twitter Account Data Analysis & Visualization Tool <3',
      author='Anjan Roy',
      author_email='anjanroy@yandex.com',
      url='https://itzmeanjan.github.io/twiz',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.7',
     )



#!/usr/bin/python3

from dataclasses import dataclass


@dataclass
class Info:
    fullName: str
    screenName: str


if __name__ == '__main__':
    print('It\'s not supposed to be used this way !')
